@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004,2005,2006,2007 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2004-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -67,6 +67,9 @@ static int iterator(idmef_value_t *val, void *extra)
         prelude_string_t *out;
         struct iterator_data *data = extra;
 
+        if ( ! val )
+                return 0;
+
         ret = prelude_string_new(&out);
         if ( ret < 0 ) {
                 prelude_perror(ret, "error creating string object");
