@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2008 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2008-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -262,6 +262,9 @@ static int iterate_cb(idmef_value_t *value, void *extra)
         int ret;
         iterate_data_t *data = extra;
 
+        if ( ! value )
+                return 0;
+
         if ( idmef_value_is_list(value) )
                 return idmef_value_iterate(value, iterate_cb, extra);
 
@@ -462,6 +465,7 @@ static int retrieve_from_db(smtp_plugin_t *plugin, const char *criteria_str)
         prelude_string_t *str;
         preludedb_result_idents_t *results;
         prelude_list_t clist, *tmp, *bkp;
+        unsigned int ident_idx = 0;
 
         ret = idmef_criteria_new_from_string(&criteria, criteria_str);
         if ( ret < 0 ) {
@@ -482,7 +486,7 @@ static int retrieve_from_db(smtp_plugin_t *plugin, const char *criteria_str)
         send_correlation_alert_notice(plugin, ret);
         prelude_list_init(&clist);
 
-        while ( preludedb_result_idents_get_next(results, &dbident) ) {
+        while ( preludedb_result_idents_get(results, ident_idx++, &dbident) ) {
                 ret = preludedb_get_alert(plugin->db, dbident, &idmef);
                 if ( ret < 0 ) {
                         prelude_log(PRELUDE_LOG_ERR, "failure retrieving message ident %" PRELUDE_PRIu64 ".\n", dbident);
@@ -925,9 +929,8 @@ static int parse_path(smtp_plugin_t *plugin, mail_format_t **fmt,
         int ret;
         size_t i = 0;
         idmef_path_t *path;
-        char path_s[1024], *ptr;
+        char path_s[1024];
 
-        ptr = path_s;
         while ( i < sizeof(path_s) - 1 &&
                 (isalnum(**in) || **in == '(' || **in == ')' || **in == '.' || **in == '-' || **in == '_' || **in == '*') ) {
                 path_s[i++] = **in;
