@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2007 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2007-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -157,7 +157,6 @@ static int flush_bufpool_to_disk(bufpool_t *bp)
 
 static bufpool_t *evict_from_memory(void)
 {
-        int ret;
         size_t prev_len = 0;
         bufpool_t *bp = NULL, *evict = NULL;
 
@@ -195,7 +194,7 @@ static bufpool_t *evict_from_memory(void)
 
 
         if ( evict ) {
-                ret = flush_bufpool_to_disk(evict);
+                flush_bufpool_to_disk(evict);
                 gl_lock_unlock(evict->mutex);
                 return evict;
         }
@@ -237,10 +236,14 @@ int bufpool_add_message(bufpool_t *bp, prelude_msg_t *msg)
 
         else {
                 ret = prelude_failover_save_msg(bp->failover, msg);
+                if ( ret < 0 )
+                        goto error;
+
                 inc_dlen(bp, prelude_msg_get_len(msg));
                 prelude_msg_destroy(msg);
         }
 
+    error:
         gl_lock_unlock(bp->mutex);
 
         return ret;
