@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004, 2005 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2004-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -29,7 +29,7 @@
 
 int manager_auth_disable_encryption(server_generic_client_t *client, prelude_io_t *pio);
 
-int manager_auth_client(server_generic_client_t *client, prelude_io_t *pio, gnutls_alert_description *alert);
+int manager_auth_client(server_generic_client_t *client, prelude_io_t *pio, gnutls_alert_description_t *alert);
 
 int manager_auth_init(prelude_client_t *client, const char *tlsopts, int dh_bits, int dh_regenerate);
 
