@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004-2007, 2008 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2004-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -31,12 +31,10 @@ typedef struct reverse_relay_receiver reverse_relay_receiver_t;
 
 void reverse_relay_set_receiver_dead(reverse_relay_receiver_t *rrr);
 
-int reverse_relay_set_receiver_alive(reverse_relay_receiver_t *rrr, server_generic_client_t *client);
+int reverse_relay_set_receiver_alive(reverse_relay_receiver_t *rrr);
 
 int reverse_relay_new_receiver(reverse_relay_receiver_t **rrr, server_generic_client_t *client, uint64_t analyzerid);
 
-reverse_relay_receiver_t *reverse_relay_search_receiver(uint64_t analyzerid);
-
 void reverse_relay_send_prepared(void);
 
 void reverse_relay_send_receiver(idmef_message_t *idmef);
