@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2001, 2002, 2003, 2004, 2005 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2001-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -27,6 +27,15 @@
 #include "idmef-message-scheduler.h"
 #include "reverse-relaying.h"
 
+
+typedef struct {
+        prelude_list_t list;
+        prelude_msg_t *msg;
+        uint32_t write_index;
+} sensor_msg_t;
+
+
+
 typedef struct {
         SERVER_GENERIC_OBJECT;
         prelude_list_t list;
@@ -35,7 +44,6 @@ typedef struct {
         prelude_connection_t *cnx;
         prelude_bool_t we_connected;
         prelude_list_t write_msg_list;
-        reverse_relay_receiver_t *rrr;
 
         uint32_t instance_id;
 } sensor_fd_t;
@@ -51,4 +59,8 @@ int sensor_server_add_client(server_generic_t *server, server_generic_client_t *
 
 int sensor_server_write_client(server_generic_client_t *dst, prelude_msg_t *msg);
 
+int sensor_server_init(void);
+
+prelude_list_t *sensor_server_get_list(uint64_t analyzerid);
+
 #endif /* _MANAGER_SENSOR_SERVER_H */
