@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2000-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -31,9 +31,8 @@
 
 #define SERVER_GENERIC_CLIENT_STATE_AUTHENTICATED  0x01
 #define SERVER_GENERIC_CLIENT_STATE_ACCEPTED       0x02
-#define SERVER_GENERIC_CLIENT_STATE_FLUSHING       0x04
-#define SERVER_GENERIC_CLIENT_STATE_CLOSING        0x08
-#define SERVER_GENERIC_CLIENT_STATE_CLOSED         0x10
+#define SERVER_GENERIC_CLIENT_STATE_CLOSING        0x04
+#define SERVER_GENERIC_CLIENT_STATE_CLOSED         0x08
 
 #ifdef HAVE_IPV6
 # define SERVER_SOCKADDR_TYPE struct sockaddr_in6
@@ -51,7 +50,7 @@
         int state;                   \
         uint64_t ident;              \
         prelude_connection_permission_t permission; \
-        gnutls_alert_description alert; \
+        gnutls_alert_description_t alert; \
         SERVER_SOCKADDR_TYPE sa; \
         server_generic_t *server
 
