@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2001-2005,2006,2007 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2001-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -30,6 +30,9 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <pathmax.h>
+#include <dirent.h>
+#include <gnutls/gnutls.h>
 #include <errno.h>
 
 #if TIME_WITH_SYS_TIME
@@ -70,8 +73,8 @@ static gnutls_priority_t tls_priority;
 
 static int global_dh_lifetime;
 static unsigned int global_dh_bits;
-static gnutls_certificate_credentials cred;
-static gnutls_dh_params cur_dh_params = NULL;
+static gnutls_certificate_credentials_t cred;
+static gnutls_dh_params_t cur_dh_params = NULL;
 static prelude_timer_t dh_param_regeneration_timer;
 static gl_lock_t dh_regen_mutex = gl_lock_initializer;
 
@@ -150,14 +153,12 @@ static int dh_check_elapsed(void)
 
 
 
-static int dh_params_load(gnutls_dh_params dh, unsigned int req_bits)
+static int dh_params_load(gnutls_dh_params_t dh, unsigned int req_bits)
 {
-        int ret;
         FILE *fd;
         ssize_t size;
-        prelude_io_t *pfd;
-        unsigned int *bits;
-        gnutls_datum prime, generator;
+        char data[8192];
+        gnutls_datum_t dh_data;
 
         fd = fopen(DH_FILENAME, "r");
         if ( ! fd ) {
@@ -167,64 +168,35 @@ static int dh_params_load(gnutls_dh_params dh, unsigned int req_bits)
                 return -1;
         }
 
-        ret = prelude_io_new(&pfd);
-        if ( ret < 0 ) {
-                fclose(fd);
+        size = fread(data, 1, sizeof(data), fd);
+        fclose(fd);
+
+        if ( size <= 0 ) {
+                prelude_log(PRELUDE_LOG_ERR, "error reading DH parameters: %s.\n", strerror(errno));
                 return -1;
         }
 
-        prelude_io_set_file_io(pfd, fd);
+        dh_data.data = (unsigned char *) data;
+        dh_data.size = size;
 
-        size = prelude_io_read_delimited(pfd, (void *) &bits);
-        if ( size < 0 || size != sizeof(*bits) ) {
-                prelude_perror(size, "error reading dh-prime length");
-                goto err;
-        }
-
-        if ( *bits != req_bits )
-                goto err;
-
-        prime.size = size = prelude_io_read_delimited(pfd, &prime.data);
+        size = gnutls_dh_params_import_pkcs3(dh, &dh_data, GNUTLS_X509_FMT_PEM);
         if ( size < 0 ) {
-                prelude_perror(size, "error reading dh-prime");
-                goto err;
+                prelude_log(PRELUDE_LOG_ERR, "error importing DH parameters: %s.\n", gnutls_strerror(errno));
+                return -1;
         }
 
-        generator.size = size = prelude_io_read_delimited(pfd, &generator.data);
-        if ( size < 0 ) {
-                prelude_perror(size, "error reading dh generator");
-                goto err;
-        }
-
-        ret = gnutls_dh_params_import_raw(dh, &prime, &generator);
-        if ( ret < 0 )
-                prelude_log(PRELUDE_LOG_WARN, "error importing Diffie-Hellman parameters: %s.\n", gnutls_strerror(ret));
-
-        free(bits);
-        free(prime.data);
-        free(generator.data);
-        prelude_io_close(pfd);
-        prelude_io_destroy(pfd);
-
-        return ret;
-
- err:
-        prelude_io_close(pfd);
-        prelude_io_destroy(pfd);
-
-        return -1;
+        return 0;
 }
 
 
 
-
-static int dh_params_save(gnutls_dh_params dh, unsigned int dh_bits)
+static int dh_params_save(gnutls_dh_params_t dh, unsigned int dh_bits)
 {
         int ret, fd;
-        prelude_io_t *pfd;
-        gnutls_datum prime, generator;
+        unsigned char buf[64 * 1024];
+        size_t size = sizeof(buf);
 
-        ret = gnutls_dh_params_export_raw(dh, &prime, &generator, NULL);
+        ret = gnutls_dh_params_export_pkcs3(dh, GNUTLS_X509_FMT_PEM, buf, &size);
         if ( ret < 0 ) {
                 prelude_log(PRELUDE_LOG_WARN, "error exporting Diffie-Hellman parameters: %s.\n", gnutls_strerror(ret));
                 return -1;
@@ -233,40 +205,26 @@ static int dh_params_save(gnutls_dh_params dh, unsigned int dh_bits)
         fd = open(DH_FILENAME, O_CREAT|O_TRUNC|O_WRONLY, S_IRUSR|S_IWUSR);
         if ( fd < 0 ) {
                 prelude_log(PRELUDE_LOG_ERR, "error opening %s for writing: %s.\n", DH_FILENAME, strerror(errno));
-                free(prime.data);
-                free(generator.data);
                 return -1;
         }
 
-        ret = prelude_io_new(&pfd);
-        if ( ret < 0 ) {
-                close(fd);
-                free(prime.data);
-                free(generator.data);
-                return -1;
-        }
+        do {
+                ret = write(fd, buf, size);
+        } while ( ret < 0 && errno == EINTR );
 
-        prelude_io_set_sys_io(pfd, fd);
-        prelude_io_write_delimited(pfd, &dh_bits, sizeof(dh_bits));
-        prelude_io_write_delimited(pfd, prime.data, prime.size);
-        prelude_io_write_delimited(pfd, generator.data, generator.size);
+        if ( ret < 0 )
+                prelude_log(PRELUDE_LOG_ERR, "error writing DH data: %s.\n", strerror(errno));
 
-        prelude_io_close(pfd);
-        prelude_io_destroy(pfd);
-
-        free(prime.data);
-        free(generator.data);
-
-        return 0;
+        close(fd);
+        return ret;
 }
 
 
 
-
 static void dh_params_regenerate(void *data)
 {
         int ret;
-        gnutls_dh_params new, tmp;
+        gnutls_dh_params_t new, tmp;
 
         /*
          * generate a new DH key.
@@ -298,10 +256,10 @@ static void dh_params_regenerate(void *data)
 
 
 
-static int get_params(gnutls_session session, gnutls_params_type type, gnutls_params_st *st)
+static int get_params(gnutls_session_t session, gnutls_params_type_t type, gnutls_params_st *st)
 {
         int ret;
-        gnutls_dh_params cpy;
+        gnutls_dh_params_t cpy;
 
         if ( type == GNUTLS_PARAMS_RSA_EXPORT )
                 return -1;
@@ -331,8 +289,8 @@ static int get_params(gnutls_session session, gnutls_params_type type, gnutls_pa
 
 
 
-static int handle_gnutls_error(prelude_io_t *pio, gnutls_session session, server_generic_client_t *client, int ret,
-                               gnutls_alert_description *alert_desc)
+static int handle_gnutls_error(prelude_io_t *pio, gnutls_session_t session, server_generic_client_t *client, int ret,
+                               gnutls_alert_description_t *alert_desc)
 {
         int level;
         const char *alert;
@@ -361,7 +319,7 @@ static int handle_gnutls_error(prelude_io_t *pio, gnutls_session session, server
         else {
                 server_generic_log_client(client, PRELUDE_LOG_WARN, "TLS error: %s.\n", gnutls_strerror(ret));
                 if ( alert_desc && (ret = gnutls_error_to_alert(ret, &level)) > 0 )
-                        *alert_desc = (gnutls_alert_description) ret;
+                        *alert_desc = (gnutls_alert_description_t) ret;
         }
 
         return -1;
@@ -369,7 +327,7 @@ static int handle_gnutls_error(prelude_io_t *pio, gnutls_session session, server
 
 
 
-static int verify_certificate(server_generic_client_t *client, gnutls_session session, gnutls_alert_description *alert)
+static int verify_certificate(server_generic_client_t *client, gnutls_session_t session, gnutls_alert_description_t *alert)
 {
         int ret;
         time_t now;
@@ -427,14 +385,14 @@ static int verify_certificate(server_generic_client_t *client, gnutls_session se
 
 
 
-static int certificate_get_peer_analyzerid(server_generic_client_t *client, gnutls_session session,
+static int certificate_get_peer_analyzerid(server_generic_client_t *client, gnutls_session_t session,
                                            uint64_t *analyzerid, prelude_connection_permission_t *permission)
 {
         int ret;
         char buf[1024];
-        gnutls_x509_crt cert;
+        gnutls_x509_crt_t cert;
         size_t size = sizeof(buf);
-        const gnutls_datum *cert_list;
+        const gnutls_datum_t *cert_list;
         unsigned int cert_list_size = 0;
 
         cert_list = gnutls_certificate_get_peers(session, &cert_list_size);
@@ -492,7 +450,7 @@ static int certificate_get_peer_analyzerid(server_generic_client_t *client, gnut
 }
 
 
-static void set_default_priority(gnutls_session session)
+static void set_default_priority(gnutls_session_t session)
 {
 #ifdef HAVE_GNUTLS_STRING_PRIORITY
                 gnutls_priority_set(session, tls_priority);
@@ -502,11 +460,11 @@ static void set_default_priority(gnutls_session session)
 }
 
 
-int manager_auth_client(server_generic_client_t *client, prelude_io_t *pio, gnutls_alert_description *alert)
+int manager_auth_client(server_generic_client_t *client, prelude_io_t *pio, gnutls_alert_description_t *alert)
 {
         int ret;
         uint64_t analyzerid;
-        gnutls_session session;
+        gnutls_session_t session;
         int fd = prelude_io_get_fd(pio);
         prelude_connection_permission_t permission;
 
@@ -572,7 +530,7 @@ int manager_auth_client(server_generic_client_t *client, prelude_io_t *pio, gnut
 int manager_auth_disable_encryption(server_generic_client_t *client, prelude_io_t *pio)
 {
         int ret;
-        gnutls_session session;
+        gnutls_session_t session;
 
         session = prelude_io_get_fdptr(pio);
 
