@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 1999-2005,2006,2007 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 1999-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -45,7 +45,6 @@
 #include "sensor-server.h"
 #include "manager-options.h"
 #include "report-plugins.h"
-#include "reverse-relaying.h"
 
 
 #define DEFAULT_MANAGER_ADDR "0.0.0.0"
@@ -123,19 +122,11 @@ static server_generic_t *add_server(void)
 }
 
 
-static void del_server(void)
-{
-        server_generic_destroy(config.server[config.nserver - 1]);
-
-        config.nserver--;
-        config.server = realloc(config.server, sizeof(*config.server) * config.nserver);
-}
-
 static int add_server_default(void)
 {
+        int ret;
         char buf[128];
         server_generic_t *server;
-        int ret, prev_family = AF_UNSPEC;
         struct addrinfo *ai, *ai_start, hints;
 
         memset(&hints, 0, sizeof(hints));
@@ -143,7 +134,7 @@ static int add_server_default(void)
         hints.ai_flags = AI_PASSIVE;
         hints.ai_socktype = SOCK_STREAM;
         hints.ai_protocol = IPPROTO_TCP;
-        hints.ai_family = PF_UNSPEC;
+        hints.ai_family = AF_UNSPEC;
 
 #ifdef AI_ADDRCONFIG
         /*
@@ -174,25 +165,10 @@ static int add_server_default(void)
 
                 ret = server_generic_bind_numeric(server, ai->ai_addr, ai->ai_addrlen, DEFAULT_MANAGER_PORT);
                 if ( ret < 0 ) {
-                        char buf[128];
-
-                        /*
-                         * More information on this at:
-                         * http://lists.debian.org/debian-ipv6/2001/01/msg00031.html
-                         */
-                        if ( prelude_error_get_code(ret) == PRELUDE_ERROR_EADDRINUSE &&
-                             prev_family != AF_UNSPEC && ai->ai_family != prev_family ) {
-                                ret = 0;
-                                del_server();
-                                continue;
-                        }
-
                         inet_ntop(ai->ai_family, prelude_sockaddr_get_inaddr(ai->ai_addr), buf, sizeof(buf));
                         prelude_perror(ret, "error initializing server on %s:%u", buf, DEFAULT_MANAGER_PORT);
                         break;
                 }
-
-                prev_family = ai->ai_family;
         }
 
         if ( config.nserver == 0 ) {
@@ -207,22 +183,6 @@ static int add_server_default(void)
 
 
 
-static int set_reverse_relay(prelude_option_t *opt, const char *arg, prelude_string_t *err, void *context)
-{
-        int ret;
-
-        if ( config.nserver == 0 ) {
-                ret = add_server_default();
-                if ( ret < 0 )
-                        return -1; /* avoid duplicate option error */
-        }
-
-        return reverse_relay_create_initiator(arg);
-}
-
-
-
-
 static int set_listen_address(prelude_option_t *opt, const char *arg, prelude_string_t *err, void *context)
 {
         int ret;
@@ -539,15 +499,6 @@ int manager_options_init(prelude_option_t *rootopt, int *argc, char **argv)
         prelude_option_add(rootopt, NULL, PRELUDE_OPTION_TYPE_CFG, 0, "sched-buffer-size",
                            NULL, PRELUDE_OPTION_ARGUMENT_REQUIRED, set_sched_buffer_size, NULL);
 
-        prelude_option_add(rootopt, &opt, PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG, 'c', "child-managers",
-                           "List of managers address:port pair where messages should be gathered from",
-                           PRELUDE_OPTION_ARGUMENT_REQUIRED, set_reverse_relay, NULL);
-        /*
-         * necessary since the reverse relay need to be setup only once one
-         * server object has been created.
-         */
-        prelude_option_set_priority(opt, PRELUDE_OPTION_PRIORITY_LAST);
-
         prelude_option_add(rootopt, NULL, PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG, 'l', "listen",
                            "Address the sensors server should listen on (addr:port)",
                            PRELUDE_OPTION_ARGUMENT_REQUIRED, set_listen_address, NULL);
@@ -560,13 +511,13 @@ int manager_options_init(prelude_option_t *rootopt, int *argc, char **argv)
 
 
         /*
-         * Some plugin might require manager_client to be already initialized,
-         * for example the relaying plugin. We need to process theses option
-         * first so that --help will be recognized even throught the initialization
-         * fail.
+         * Some plugins might require manager_client to be already initialized,
+         * for example the relaying plugin. We need to process these options
+         * first so that --help will be recognized even though the initialization
+         * fails.
          *
          * We can't delay the error checking of manager_client initialization either since
-         * prelude_client_init() also need to know the configuration file that will be used.
+         * prelude_client_init() also needs to know the configuration file that will be used.
          */
 
         prelude_option_set_warnings(0, &old_warnings);
