@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 1998-2007,2008 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 1998-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -47,7 +47,7 @@
 
 #define MANAGER_MODEL "Prelude Manager"
 #define MANAGER_CLASS "Concentrator"
-#define MANAGER_MANUFACTURER "http://www.prelude-ids.com"
+#define MANAGER_MANUFACTURER "http://www.prelude-siem.com"
 #define DEFAULT_ANALYZER_NAME "prelude-manager"
 
 extern manager_config_t config;
@@ -142,7 +142,7 @@ static void sig_cb(struct ev_loop *loop, struct ev_signal *s, int revent)
 #endif
 
         handle_signal(s->signum);
-        ev_unloop(manager_event_loop, EVUNLOOP_ALL);
+        ev_break(manager_event_loop, EVUNLOOP_ALL);
 
         return;
 }
@@ -175,7 +175,7 @@ int main(int argc, char **argv)
 
         prelude_init(&argc, argv);
 
-        manager_event_loop = ev_default_loop_init(EVFLAG_AUTO);
+        manager_event_loop = ev_default_loop(EVFLAG_AUTO);
         if ( ! manager_event_loop ) {
                 prelude_log(PRELUDE_LOG_ERR, "error initializing libev.\n");
                 return -1;
@@ -220,6 +220,7 @@ int main(int argc, char **argv)
                 return -1;
         prelude_log(PRELUDE_LOG_DEBUG, "Initialized %d filtering plugins.\n", ret);
 
+        sensor_server_init();
 
         ret = manager_options_init(manager_root_optlist, &argc, argv);
         if ( ret < 0 )
