@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004-2007,2008 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2004-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -47,65 +47,33 @@
 
 struct reverse_relay_receiver {
         prelude_list_t list;
-
-
         uint64_t analyzerid;
-
-        unsigned int count;
-
         prelude_failover_t *failover;
-        server_generic_client_t *client;
 };
 
 
-typedef struct {
-        prelude_list_t list;
-
-        uint64_t analyzerid;
-        prelude_msg_t *msg;
-} mqueue_t;
-
 
 static prelude_msgbuf_t *msgbuf;
 static PRELUDE_LIST(mqueue_list);
 static gl_lock_t mqueue_mutex = gl_lock_initializer;
 
-
-static PRELUDE_LIST(receiver_list);
+static prelude_bool_t no_receiver = TRUE;
+static prelude_list_t receiver_list[1024];
 static gl_lock_t receiver_list_mutex = gl_lock_initializer;
 
 extern manager_config_t config;
 extern prelude_client_t *manager_client;
-static prelude_connection_pool_t *initiator = NULL;
 
 
-static int connection_event_cb(prelude_connection_pool_t *pool,
-                               prelude_connection_pool_event_t event, prelude_connection_t *cnx)
+
+static unsigned int get_list_key(uint64_t analyzerid)
 {
-        int ret;
-        server_generic_client_t *client;
-
-        if ( ! (event & PRELUDE_CONNECTION_POOL_EVENT_ALIVE) )
-                return 0;
-
-#if (defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__
-        ret = fcntl(prelude_io_get_fd(prelude_connection_get_fd(cnx)), F_SETFL, O_NONBLOCK);
-        if ( ret < 0 )
-                return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "could not set non blocking mode for client: %s", strerror(errno));
-#endif
-
-        ret = sensor_server_add_client(config.server[0], &client, cnx);
-        if ( ret < 0 )
-                prelude_log(PRELUDE_LOG_WARN, "error adding new client to reverse relay list.\n");
-
-        prelude_connection_set_data(cnx, client);
-
-        return 0;
+        return analyzerid & (sizeof(receiver_list) / sizeof(*receiver_list) - 1);
 }
 
 
 
-static reverse_relay_receiver_t *get_next_receiver(prelude_list_t **iter)
+static reverse_relay_receiver_t *get_next_receiver(prelude_list_t *receiver_list, prelude_list_t **iter)
 {
         prelude_list_t *tmp;
         reverse_relay_receiver_t *rrr = NULL;
@@ -115,7 +83,7 @@ static reverse_relay_receiver_t *get_next_receiver(prelude_list_t **iter)
          * modified by the worker thread. We only protect writing
          * of the list, and reading it from the worker thread.
          */
-        prelude_list_for_each_continue_safe(&receiver_list, tmp, *iter) {
+        prelude_list_for_each_continue_safe(receiver_list, tmp, *iter) {
                 rrr = prelude_list_entry(tmp, reverse_relay_receiver_t, list);
                 break;
         }
@@ -125,70 +93,69 @@ static reverse_relay_receiver_t *get_next_receiver(prelude_list_t **iter)
 
 
 
+static int write_all_client(reverse_relay_receiver_t *rrr, prelude_msg_t *msg, unsigned long count)
+{
+        int i = 0, ret;
+        sensor_fd_t *client;
+        prelude_list_t *tmp;
 
-int reverse_relay_set_receiver_alive(reverse_relay_receiver_t *rrr, server_generic_client_t *client)
+        prelude_list_for_each(sensor_server_get_list(rrr->analyzerid), tmp) {
+                client = prelude_list_entry(tmp, sensor_fd_t, list);
+
+                if ( client->ident != rrr->analyzerid )
+                        continue;
+
+                i++;
+                if ( count )
+                        server_generic_log_client((server_generic_client_t *) client, PRELUDE_LOG_INFO,
+                                                  "flushing %lu messages received while analyzer was offline.\n", count);
+
+                ret = sensor_server_write_client((server_generic_client_t *) client, prelude_msg_ref(msg));
+                if ( ret < 0 && prelude_error_get_code(ret) != PRELUDE_ERROR_EAGAIN )
+                        prelude_msg_destroy(msg);
+        }
+
+        return i;
+}
+
+
+
+int reverse_relay_set_receiver_alive(reverse_relay_receiver_t *rrr)
 {
         ssize_t size;
-        int ret, state;
         prelude_msg_t *msg;
         prelude_failover_t *failover = rrr->failover;
+        unsigned long avail = prelude_failover_get_available_msg_count(failover);
 
-        size = prelude_failover_get_saved_msg(failover, &msg);
-        if ( size == 0 )
-                rrr->client = client;
+        while ( (size = prelude_failover_get_saved_msg(failover, &msg)) > 0 ) {
+                write_all_client(rrr, msg, avail);
+                prelude_msg_destroy(msg);
+
+                avail = 0;
+        }
 
         if ( size < 0 ) {
                 prelude_perror((prelude_error_t) size, "could not retrieve saved message from disk");
                 return -1;
         }
 
-        if ( size > 0 ) {
-                rrr->count++;
-
-                state = server_generic_client_get_state(client);
-                if ( ! (state & SERVER_GENERIC_CLIENT_STATE_FLUSHING) )
-                        server_generic_client_set_state(client, state | SERVER_GENERIC_CLIENT_STATE_FLUSHING);
-
-                ret = sensor_server_write_client(client, msg);
-                if ( ret < 0 ) {
-                        if ( prelude_error_get_code(ret) == PRELUDE_ERROR_EAGAIN )
-                                return 0;
-
-                        return ret;
-                }
-
-                server_generic_notify_write_enable(client);
-                return 1;
-        }
-
-        if ( rrr->count ) {
-                server_generic_log_client(client, PRELUDE_LOG_INFO,
-                                          "flushed %u messages received while analyzer was offline.\n", rrr->count);
-                rrr->count = 0;
-        }
-
-        server_generic_client_set_state(client, server_generic_client_get_state(client) & ~SERVER_GENERIC_CLIENT_STATE_FLUSHING);
-
         return 0;
 }
 
 
 
-int reverse_relay_set_initiator_dead(prelude_connection_t *cnx)
+static reverse_relay_receiver_t *reverse_relay_search_receiver(uint64_t analyzerid)
 {
-        int ret = -1;
+        prelude_list_t *iter = NULL;
+        reverse_relay_receiver_t *item;
 
-        if ( initiator )
-                ret = prelude_connection_pool_set_connection_dead(initiator, cnx);
+        while ( (item = get_next_receiver(&receiver_list[get_list_key(analyzerid)], &iter)) ) {
 
-        return ret;
-}
+                if ( item->analyzerid == analyzerid )
+                        return item;
+        }
 
-
-
-void reverse_relay_set_receiver_dead(reverse_relay_receiver_t *rrr)
-{
-        rrr->client = NULL;
+        return NULL;
 }
 
 
@@ -199,14 +166,15 @@ int reverse_relay_new_receiver(reverse_relay_receiver_t **rrr, server_generic_cl
         char fname[PATH_MAX];
         reverse_relay_receiver_t *new;
 
+        new = reverse_relay_search_receiver(analyzerid);
+        if ( new )
+                goto out;
+
         new = malloc(sizeof(*new));
         if ( ! new )
                 return -1;
 
-        new->count = 0;
-        new->client = client;
         new->analyzerid = analyzerid;
-
         prelude_client_profile_get_backup_dirname(prelude_client_get_profile(manager_client), fname, sizeof(fname));
         snprintf(fname + strlen(fname), sizeof(fname) - strlen(fname), "/%" PRELUDE_PRIu64, analyzerid);
 
@@ -218,45 +186,23 @@ int reverse_relay_new_receiver(reverse_relay_receiver_t **rrr, server_generic_cl
         }
 
         gl_lock_lock(receiver_list_mutex);
-        prelude_list_add_tail(&receiver_list, &new->list);
+        no_receiver = FALSE;
+        prelude_list_add_tail(&receiver_list[get_list_key(analyzerid)], &new->list);
         gl_lock_unlock(receiver_list_mutex);
         *rrr = new;
 
-        return 0;
+    out:
+        return reverse_relay_set_receiver_alive(new);
 }
 
 
 
-reverse_relay_receiver_t *reverse_relay_search_receiver(uint64_t analyzerid)
-{
-        prelude_list_t *iter = NULL;
-        reverse_relay_receiver_t *item;
-
-        while ( (item = get_next_receiver(&iter)) ) {
-
-                if ( analyzerid == item->analyzerid )
-                        return item;
-        }
-
-        return NULL;
-}
-
-
 static int send_msgbuf(prelude_msgbuf_t *msgbuf, prelude_msg_t *msg)
 {
-        mqueue_t *mq;
-
-        mq = malloc(sizeof(*mq));
-        if ( ! mq ) {
-                prelude_log(PRELUDE_LOG_ERR, "memory exhausted: %s.\n", strerror(errno));
-                return -1;
-        }
-
-        mq->msg = msg;
-        mq->analyzerid = *(uint64_t *) prelude_msgbuf_get_data(msgbuf);
+        prelude_msg_set_data(msg, prelude_msgbuf_get_data(msgbuf));
 
         gl_lock_lock(mqueue_mutex);
-        prelude_list_add_tail(&mqueue_list, &mq->list);
+        prelude_linked_object_add_tail(&mqueue_list, (prelude_linked_object_t *) msg);
         gl_lock_unlock(mqueue_mutex);
 
         return 0;
@@ -307,21 +253,21 @@ static int get_issuer_analyzerid(idmef_message_t *idmef, uint64_t *analyzerid)
 
 
 
-static mqueue_t *mqueue_get_next(void)
+static prelude_msg_t *mqueue_get_next(void)
 {
-        mqueue_t *q = NULL;
+        prelude_msg_t *m = NULL;
 
         gl_lock_lock(mqueue_mutex);
 
         if ( prelude_list_is_empty(&mqueue_list) )
                 goto out;
 
-        q = prelude_list_entry(mqueue_list.next, mqueue_t, list);
-        prelude_list_del(&q->list);
+        m = prelude_linked_object_get_object(mqueue_list.next);
+        prelude_linked_object_del((prelude_linked_object_t *) m);
 
 out:
         gl_lock_unlock(mqueue_mutex);
-        return q;
+        return m;
 }
 
 
@@ -330,16 +276,12 @@ void reverse_relay_send_receiver(idmef_message_t *idmef)
 {
         int ret;
         uint64_t analyzerid;
-        prelude_bool_t empty;
+        reverse_relay_receiver_t *rrr;
 
         /*
          * If there is no receiver, no need to queue the message.
          */
-        gl_lock_lock(receiver_list_mutex);
-        empty = prelude_list_is_empty(&receiver_list);
-        gl_lock_unlock(receiver_list_mutex);
-
-        if ( empty )
+        if ( no_receiver )
                 return;
 
         /*
@@ -351,13 +293,15 @@ void reverse_relay_send_receiver(idmef_message_t *idmef)
         if ( ret < 0 )
                 return;
 
+        rrr = reverse_relay_search_receiver(analyzerid);
+        prelude_msgbuf_set_data(msgbuf, rrr);
+
+
         /*
-         * Convert from idmef_message_t to prelude_msg_t,
-         * this will trigger the msgbuf callback where an mqueue_t
-         * object will be created, and attached to the list of message
-         * to be emited.
+         * Convert from idmef_message_t to prelude_msg_t, this will
+         * trigger the msgbuf callback where the message will be attached
+         * to the list of message to be emited.
          */
-        prelude_msgbuf_set_data(msgbuf, &analyzerid);
         idmef_message_write(idmef, msgbuf);
         prelude_msgbuf_mark_end(msgbuf);
 
@@ -373,95 +317,41 @@ void reverse_relay_send_receiver(idmef_message_t *idmef)
 
 void reverse_relay_send_prepared(void)
 {
-        int ret;
-        mqueue_t *mq;
-        prelude_list_t *iter = NULL;
+        int ret, i;
+        prelude_msg_t *m;
+        prelude_list_t *tmp;
         reverse_relay_receiver_t *receiver;
 
-        while ( (mq = mqueue_get_next()) ) {
+        while ( (m = mqueue_get_next()) ) {
 
-                while ( (receiver = get_next_receiver(&iter)) ) {
+                for ( i = 0; i < sizeof(receiver_list) / sizeof(*receiver_list); i++ ) {
+                        prelude_list_for_each(&receiver_list[i], tmp) {
+                                receiver = prelude_list_entry(tmp, reverse_relay_receiver_t, list);
 
-                        if ( mq->analyzerid == receiver->analyzerid )
-                                continue;
+                                if ( prelude_msg_get_data(m) == receiver )
+                                        continue;
 
-                        if ( receiver->client )
-                                sensor_server_write_client(receiver->client, prelude_msg_ref(mq->msg));
-                        else {
-                                ret = prelude_failover_save_msg(receiver->failover, mq->msg);
-                                if ( ret < 0 )
-                                        prelude_perror(ret, "could not save message to disk");
+                                ret = write_all_client(receiver, m, 0);
+                                if ( ! ret ) {
+                                        ret = prelude_failover_save_msg(receiver->failover, m);
+                                        if ( ret < 0 )
+                                                prelude_perror(ret, "could not save message to disk");
+                                }
                         }
                 }
 
-                prelude_msg_destroy(mq->msg);
-                free(mq);
+                prelude_msg_destroy(m);
         }
 }
 
 
-static void destroy_current_initiator(void)
-{
-        sensor_fd_t *client;
-        prelude_list_t *tmp;
-        prelude_connection_t *cnx;
-
-        prelude_list_for_each(prelude_connection_pool_get_connection_list(initiator), tmp) {
-                cnx = prelude_linked_object_get_object(tmp);
-
-                client = prelude_connection_get_data(cnx);
-                if ( client ) {
-                        client->cnx = NULL;
-                        client->fd = NULL;
-                        server_generic_remove_client(config.server[0], (server_generic_client_t *) client);
-                }
-        }
-
-        prelude_connection_pool_destroy(initiator);
-        initiator = NULL;
-}
-
-
-
-int reverse_relay_create_initiator(const char *arg)
-{
-        int ret;
-        prelude_client_profile_t *cp;
-
-        cp = prelude_client_get_profile(manager_client);
-
-        if ( initiator )
-                destroy_current_initiator();
-
-        ret = prelude_connection_pool_new(&initiator, cp, PRELUDE_CONNECTION_PERMISSION_IDMEF_READ);
-        if ( ret < 0 )
-                goto out;
-
-        prelude_connection_pool_set_flags(initiator, PRELUDE_CONNECTION_POOL_FLAGS_RECONNECT);
-        prelude_connection_pool_set_event_handler(initiator, PRELUDE_CONNECTION_POOL_EVENT_DEAD |
-                                                  PRELUDE_CONNECTION_POOL_EVENT_ALIVE, connection_event_cb);
-
-        ret = prelude_connection_pool_set_connection_string(initiator, arg);
-        if ( ret < 0 ) {
-                prelude_connection_pool_destroy(initiator);
-                goto out;
-        }
-
-        ret = prelude_connection_pool_init(initiator);
-        if ( ret < 0 ) {
-                prelude_connection_pool_destroy(initiator);
-                goto out;
-        }
-
- out:
-        return ret;
-}
-
-
 
 int reverse_relay_init(void)
 {
-        int ret;
+        int ret, i;
+
+        for ( i = 0; i < sizeof(receiver_list) / sizeof(*receiver_list); i++ )
+                prelude_list_init(&receiver_list[i]);
 
         ret = prelude_msgbuf_new(&msgbuf);
         if ( ! msgbuf ) {
