@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2001-2007,2008 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2001-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -53,8 +53,17 @@
 
 extern prelude_client_t *manager_client;
 
-static PRELUDE_LIST(sensors_cnx_list);
+
 static uint32_t global_instance_id = 0;
+static prelude_list_t sensors_cnx_list[1024];
+
+
+
+static unsigned int get_list_key(uint64_t analyzerid)
+{
+        return analyzerid & (sizeof(sensors_cnx_list) / sizeof(*sensors_cnx_list) - 1);
+}
+
 
 
 static sensor_fd_t *search_client(prelude_list_t *head, uint64_t analyzerid, uint32_t instance_id)
@@ -74,24 +83,57 @@ static sensor_fd_t *search_client(prelude_list_t *head, uint64_t analyzerid, uin
 
 
 
-static int write_client(sensor_fd_t *dst, prelude_msg_t *msg)
+static int queue_write_client(sensor_fd_t *client, prelude_msg_t *msg, uint32_t windex)
 {
-        int ret;
+        sensor_msg_t *smsg;
 
-        ret = prelude_msg_write(msg, dst->fd);
+        smsg = malloc(sizeof(*smsg));
+        if ( ! smsg )
+                return prelude_error_from_errno(errno);
+
+        smsg->write_index = windex;
+        smsg->msg = msg;
+
+        if ( windex )
+                prelude_list_add(&client->write_msg_list, &smsg->list);
+        else
+                prelude_list_add_tail(&client->write_msg_list, &smsg->list);
+
+        return 0;
+}
+
+
+
+static int write_client(sensor_fd_t *dst, sensor_msg_t *cont, prelude_msg_t *msg)
+{
+        int ret, ret2;
+        uint32_t local_windex = 0, *windex;
+
+        windex = (cont) ? &cont->write_index : &local_windex;
+
+        ret = prelude_msg_write_r(msg, dst->fd, windex);
         if ( ret < 0 && prelude_error_get_code(ret) == PRELUDE_ERROR_EAGAIN ) {
+                if ( cont )
+                        prelude_list_add(&dst->write_msg_list, &cont->list);
+                else {
+                        ret2 = queue_write_client(dst, msg, local_windex);
+                        if ( ret2 < 0 )
+                                return ret2;
+                }
 
-                prelude_linked_object_add(&dst->write_msg_list, (prelude_linked_object_t *) msg);
                 server_generic_notify_write_enable((server_generic_client_t *) dst);
-
                 return ret;
         }
 
         if ( ret != 0 )
                 prelude_log(PRELUDE_LOG_ERR, "could not write msg: %s.\n", prelude_strerror(ret));
 
-        prelude_msg_destroy(msg);
+        if ( cont ) {
+                prelude_list_del(&cont->list);
+                free(cont);
+        }
 
+        prelude_msg_destroy(msg);
         return ret;
 }
 
@@ -100,12 +142,9 @@ static int write_client(sensor_fd_t *dst, prelude_msg_t *msg)
 static int forward_message_to_analyzerid(sensor_fd_t *client, uint64_t analyzerid, uint32_t instance_no, prelude_msg_t *msg)
 {
         int ret = 0;
-        uint8_t tag;
         sensor_fd_t *target;
 
-        tag = prelude_msg_get_tag(msg);
-
-        target = search_client(&sensors_cnx_list, analyzerid, instance_no);
+        target = search_client(&sensors_cnx_list[get_list_key(analyzerid)], analyzerid, instance_no);
         if ( ! target )
                 return -1;
 
@@ -245,6 +284,7 @@ static int send_unreachable_message(server_generic_client_t *client, uint64_t *i
 static int process_request_cb(prelude_msgbuf_t *msgbuf, prelude_msg_t *msg)
 {
         int ret;
+
         ret = sensor_server_write_client(prelude_msgbuf_get_data(msgbuf), msg);
         if ( ret < 0 && prelude_error_get_code(ret) == PRELUDE_ERROR_EAGAIN )
                 return 0; /* message is queued */
@@ -361,28 +401,13 @@ static int reply_sensor_option(sensor_fd_t *client, prelude_msg_t *msg)
 
 static int handle_declare_receiver(sensor_fd_t *sclient)
 {
-        int ret;
+        reverse_relay_receiver_t *rrr;
         server_generic_client_t *client = (server_generic_client_t *) sclient;
 
         if ( ! sclient->ident )
                 return -1;
 
-        sclient->rrr = reverse_relay_search_receiver(sclient->ident);
-        if ( ! sclient->rrr ) {
-                /*
-                 * First time a child relay with this address connect here.
-                 * Add it to the manager list. Type of the created connection is -parent-
-                 * because *we* are sending the alert to the child.
-                 */
-                ret = reverse_relay_new_receiver(&sclient->rrr, client, sclient->ident);
-                if ( ret < 0 )
-                        return ret;
-        }
-
-        server_generic_log_client(client, PRELUDE_LOG_INFO,
-                                  "client requested forward of IDMEF message.\n");
-
-        return reverse_relay_set_receiver_alive(sclient->rrr, client);
+        return reverse_relay_new_receiver(&rrr, client, sclient->ident);
 }
 
 
@@ -395,7 +420,7 @@ static int handle_declare_client(sensor_fd_t *cnx)
                 return -1;
 
         cnx->instance_id = ++global_instance_id;
-        prelude_list_add_tail(&sensors_cnx_list, &cnx->list);
+        prelude_list_add_tail(&sensors_cnx_list[get_list_key(cnx->ident)], &cnx->list);
 
         return 0;
 }
@@ -514,88 +539,40 @@ static int read_connection_cb(server_generic_client_t *client)
 
 static int write_connection_cb(server_generic_client_t *client)
 {
-        int ret = 1;
-        prelude_list_t *tmp;
-        prelude_msg_t *cur = NULL;
+        int ret = 0;
+        size_t limit = 1024; /* FIXME: hardcoded limit */
+        prelude_list_t *tmp, *bkp;
+        sensor_msg_t *cur = NULL;
         sensor_fd_t *sclient = (sensor_fd_t *) client;
 
-        prelude_list_for_each(&sclient->write_msg_list, tmp) {
-                cur = prelude_linked_object_get_object(tmp);
-                prelude_linked_object_del((prelude_linked_object_t *) cur);
+        prelude_list_for_each_safe(&sclient->write_msg_list, tmp, bkp) {
+                cur = prelude_list_entry(tmp, sensor_msg_t, list);
+                prelude_list_del(&cur->list);
 
-                ret = write_client(sclient, cur);
+                ret = write_client(sclient, cur, cur->msg);
                 if ( ret < 0 && prelude_error_get_code(ret) == PRELUDE_ERROR_EAGAIN ) {
                         ret = 0;
-                        goto out;
+                        break;
                 }
 
-                break;
+                if ( limit-- == 0 )
+                        break;
         }
 
-        if ( prelude_list_is_empty(&sclient->write_msg_list) ) {
+        if ( prelude_list_is_empty(&sclient->write_msg_list) )
                 server_generic_notify_write_disable(client);
-                if ( server_generic_client_get_state(client) & SERVER_GENERIC_CLIENT_STATE_FLUSHING )
-                        ret = reverse_relay_set_receiver_alive(sclient->rrr, client);
-                else
-                        ret = 0;
-        }
 
-out:
-        return ret; /* We yet have other message to process */
-}
-
-
-
-static int do_close_cnx(server_generic_client_t *ptr, sensor_fd_t *cnx)
-{
-        int ret;
-        void *fd_ptr;
-        prelude_error_code_t code;
-
-         do {
-                 ret = prelude_connection_close(cnx->cnx);
-                 if ( ret == 0 )
-                         break;
-
-                 code = prelude_error_get_code(ret);
-                 if ( code == PRELUDE_ERROR_EAGAIN ) {
-
-                         fd_ptr = prelude_io_get_fdptr(prelude_connection_get_fd(cnx->cnx));
-                         if ( fd_ptr && gnutls_record_get_direction(fd_ptr) == 1 )
-                                 server_generic_notify_write_enable(ptr);
-
-                         return -1;
-                 }
-
-                 server_generic_log_client(ptr, PRELUDE_LOG_WARN, "%s.\n", prelude_strerror(ret));
-
-         } while ( ret < 0 && ! prelude_io_is_error_fatal(prelude_connection_get_fd(cnx->cnx), ret));
-
-         return 0;
+        return ret;
 }
 
 
 
 static int close_connection_cb(server_generic_client_t *ptr)
 {
-        int ret;
         prelude_msg_t *msg;
         prelude_list_t *tmp, *bkp;
         sensor_fd_t *cnx = (sensor_fd_t *) ptr;
 
-        if ( cnx->rrr )
-                reverse_relay_set_receiver_dead(cnx->rrr);
-
-        else if ( cnx->cnx ) {
-                cnx->fd = NULL;
-                reverse_relay_set_initiator_dead(cnx->cnx);
-
-                ret = do_close_cnx(ptr, cnx);
-                if ( ret < 0 )
-                        return -1;
-        }
-
-
         if ( ! prelude_list_is_empty(&cnx->list) )
                 prelude_list_del(&cnx->list);
 
@@ -686,7 +663,6 @@ int sensor_server_add_client(server_generic_t *server, server_generic_client_t *
         cdata->fd = prelude_connection_get_fd(cnx);
 
         cdata->cnx = cnx;
-        cdata->rrr = NULL;
         cdata->we_connected = TRUE;
 
         cdata->server = server;
@@ -695,23 +671,41 @@ int sensor_server_add_client(server_generic_t *server, server_generic_client_t *
         cdata->ident = prelude_connection_get_peer_analyzerid(cnx);
 
         server_generic_client_set_permission((server_generic_client_t *)cdata, prelude_connection_get_permission(cnx));
-        prelude_list_add(&sensors_cnx_list, &cdata->list);
+        prelude_list_add(&sensors_cnx_list[get_list_key(cdata->ident)], &cdata->list);
 
         return server_generic_process_requests(server, (server_generic_client_t *) cdata);
 }
 
 
+
 int sensor_server_write_client(server_generic_client_t *client, prelude_msg_t *msg)
 {
         int ret;
         sensor_fd_t *dst = (sensor_fd_t *) client;
 
         if ( prelude_list_is_empty(&dst->write_msg_list) )
-                ret = write_client(dst, msg);
-        else {
-                ret = 0;
-                prelude_linked_object_add_tail(&dst->write_msg_list, (prelude_linked_object_t *) msg);
-        }
+                ret = write_client(dst, NULL, msg);
+        else
+                ret = queue_write_client(dst, msg, 0);
 
         return ret;
 }
+
+
+
+int sensor_server_init(void)
+{
+        int i;
+
+        for ( i = 0; i < sizeof(sensors_cnx_list) / sizeof(*sensors_cnx_list); i++ )
+                prelude_list_init(&sensors_cnx_list[i]);
+
+        return 0;
+}
+
+
+
+prelude_list_t *sensor_server_get_list(uint64_t analyzerid)
+{
+        return &sensors_cnx_list[get_list_key(analyzerid)];
+}
